{% extends "admin/cms/page/change_form.html" %}
{% load i18n admin_urls admin_static admin_modify cms_admin cms_static %}

{% block content_title %}
    <h1>{% block title %}{% if add %}{% trans "Add an article" %}{% else %}{% trans "Change an article" %}{% endif %}{% endblock %}</h1>
{% endblock %}

{% block content %}

{% with add=True %}
{{ block.super }}
{% endwith %}

{% endblock %}
