th.column-view,
th[class*="column-lang_"],
th[class="column-preview_link"] {
    width: 1%;
    text-align: center;
}
td.field-view,
td[class*="field-lang_"] {
    text-align: center;
}
.sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0,0,0,0);
  border: 0;
}
.cms-pagetree-dropdown-menu-open {
    display: block;
}
.cms-hover-tooltip {z-index: auto;}

/* fix conflicts with djangocms-admin.css */
.change-list table td .cms-pagetree-dropdown-menu a:not(.cke_button) {
    line-height: 1.5;
    padding: 10px 15px;
}
.change-list table td .cms-tree-item-preview a:not(.cke_button) {
    font-size: 18px !important;
    line-height: 18px;
}
